;; Generated package description from uni-confusables.el  -*- no-byte-compile: t -*-
(define-package "uni-confusables" "0.2.0.20201128.224446" "Unicode confusables table" 'nil :url "https://elpa.gnu.org/packages/uni-confusables.html" :maintainer '("Teodor Zlatanov" . "tzz@lifelogs.com"))
